
package org.mdanderson.biostat.geneclust;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.text.MessageFormat;
import java.util.ResourceBundle;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

final class SBufferedWriter extends BufferedWriter
{
    public SBufferedWriter (OutputStreamWriter osw, int bufferSize)
    {
	super (osw, bufferSize);
    }

    public void SetStringVar (String var, String val)
	throws java.io.IOException
    {
        write(var + "<-" + "\"" + val + "\";");
        newLine();
    }

    public void SetLogicalVar (String var, boolean val)
	throws java.io.IOException
    {
	if (val == true)
            write(var + "<-" + "TRUE" + ";");
	else
            write(var + "<-" + "FALSE" + ";");
        newLine();
    }

    public void SetIntegerVar (String var, int val)
	throws java.io.IOException
    {
        write(var + "<-" + val + ";");
        newLine();
    }

    public void Command (String cmd)
	throws java.io.IOException
    {
        write(cmd);
        newLine();
    }

    public void Source (String script)
	throws java.io.IOException
    {
        write("source(paste(GCinstall, " +
                               "\"sinterpreter\", " +
                               "\"" + script +
                               "\", sep=\"/\"))");
        newLine();
    }

    /**
     * Writes a 'header' with the current time in ISO8601 format
     * to the output stream.
     *
     * @param bw  output stream writer
     * @throws IOException if an error occurs while writing
     */
    protected void TimeStamp (String msg)
      throws java.io.IOException
    {
        final String iso8601DateFormat = "yyyy-MM-dd'T'hh:mm:ss'Z'";
        DateFormat df = new SimpleDateFormat(iso8601DateFormat);

        df.setTimeZone(TimeZone.getTimeZone("UTC"));
        String dateString = df.format(new Date());

        write("# " + msg + "  ---  " + dateString);
        newLine();
    }

}
