/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for pathname textfield validation class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)PathnameValidator.java    1.0 2002/03/18
 *
 * Copyright (c) 2002 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.io.File;


/**
 * PathnameValidator - Implements pathname validation.
 *
 * @version 1.0 2002/03/18
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class PathnameValidator extends ValidationAdapter
{
    // =========================================================================
    // IMPLEMENTS VALIDATOR
    // =========================================================================

    /**
     * Determines the validity of pathname data. Converts the value to an
     * abstract pathname and ensures it exists.
     *
     * @param value  object with data to be checked
     * @return       true if data is valid; otherwise, false
     */
    public boolean isValid(Object value)
    {
        boolean valid = false;

        if (value != null)
        {
            try
            {
                File file = new File(value.toString());

                valid = file.exists();
            }
            catch (SecurityException se)
            {
                // Consider this is as good as not existing
            }
        }

        return valid;
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Convenience function that determines the validity of pathname data
     * without the caller having to have an instance of this class available.
     *
     * @param value  data to be checked
     * @return       true if data is valid; otherwise, false
     * @see #isValid
     */
    public static boolean accepts(Object value)
    {
        return new PathnameValidator().isValid(value);
    }
}

