/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for text field validation interface
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)Validator.java    1.0 2000/12/27
 *
 * Copyright (c) 2000 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


/**
 * Validator - Interface for validation.
 *
 * @version 1.0 2000/12/27
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public interface Validator extends DocumentListener
{
    /**
     * Determines the validity of data
     *
     * @param value  data to be checked
     * @return       true if data is valid; otherwise, false
     */
    public boolean isValid(Object value);

    /**
     * Determines the validity of data and fires an event if state has changed
     *
     * @param value  data to be checked
     * @return       true if data is valid; otherwise, false
     * @see #isValid
     */
    public boolean validate(Object value);

    // =========================================================================
    // IMPLEMENT DOCUMENTLISTENER
    // =========================================================================

    /**
     * Gives notification that an attribute or set of attributes changed
     *
     * @param event  document change notification event
     */
    public void changedUpdate(DocumentEvent event);

    /**
     * Gives notification that there was an insert into the document
     *
     * @param event  document change notification event
     */
    public void insertUpdate(DocumentEvent event);

    /**
     * Gives notification that a portion of the document has been removed
     *
     * @param event  document change notification event
     */
    public void removeUpdate(DocumentEvent event);

    // =========================================================================
    // IMPLEMENT STATUSSOURCE
    // =========================================================================

    /**
     * Adds a status listener for notification of state of data changes
     *
     * @param listener  status listener
     */
    public void addStatusListener(StatusListener listener);

    /**
     * Removes a status listener
     *
     * @param listener  status listener
     */
    public void removeStatusListener(StatusListener listener);
}

