
NewGraph.func <- function (windowTitle, fileName)
{
    if (displayName == "X11")
        Motif (windowTitle)
    else
        postscript (file=fileName, title=windowTitle, paper="letter", horizontal=FALSE);
}

##******************************************************************************
PlotGap.func<-function(MyGapFile)
{
    MyFrame<-read.table(MyGapFile,header=FALSE);
    x<-MyFrame$V1;
    y<-MyFrame$V2;

    plot(x,y,col=1,pch=5,log="x",
         xlab="Cluster Size",ylab="Gap Curve");

    nperm <- ncol (MyFrame) - 2;
    for (i in 1:nperm)
        points(x,MyFrame[,i+2],col=3,type="l");

    return();
}


##******************************************************************************
PlotPC.func<-function(MyPCFile)
{
    MyFrame<-read.table(MyPCFile,header=FALSE);
    y<-MyFrame$V1;
    x<-c(1:length(y));

    plot(x,y,col=1,pch=5,
         xlab="Sample",ylab="Principal Component");

    return();
}


##*****************************************************************************
PlotVariance.func<-function(MyVarFile)
{
    MyFrame<-read.table(MyVarFile,header=FALSE);
    x<-MyFrame$V1;
    y<-MyFrame$V2;
    z<-MyFrame$V3;

    plot(x,y,col=1,pch=0,log="x",mkh=0.1,
         xlab="Cluster Size",ylab="Variance Ratio",ylim=c(0,100));
    points(x,z,col=2,pch=1,mkh=0.1);

    nperm <- ncol (MyFrame) - 6;
    for (i in 1:nperm) {
       points(x,MyFrame[,i+6],col=3,type="l");
    }
    return();
}


##******************************************************************************
Hclust.func<-function(K,identifier,fname)
{
    basetitle<-paste("Hierarchical Clustering of",identifier,sep=" ");
    if (GCinterpreter == "R") {
        dd<-dist(K,method="euclidean");
        htemp<-hclust(dd,method="average");
    }
    else {
        dd<-dist(K,metric="euclidean");
        htemp<-hclust(dd,method="connected");
    }
    cnames<-c(1:dim(K)[1]);


    outgraph<-paste(GCoutput,fname,sep="/");
    windowtitle = paste ("GeneClust", basetitle, sep=" : ");
    if (GCinterpreter == "R") {
	NewGraph.func (windowtitle, outgraph);
	# Print the hierarchical clustering graph
	hclstgraph<-plclust(htemp,unit=TRUE,level=TRUE,labels=cnames);
	maintitle<-paste(basetitle,"\n",sep="");
	title(maintitle);
    }
    else {
	Motif(windowtitle);
	# Print the hierarchical clustering graph
	hclstgraph<-plclust(htemp,cex=0.7,unit=TRUE,level=TRUE,labels=cnames);
	maintitle<-paste(basetitle,"\n",sep="");
	title(maintitle);
        printgraph(horizontal=TRUE,print=FALSE,file=outgraph);
    }

    return();
}


##******************************************************************************
DisplayHclust.func<-function(K)
{
    if (graphGenes || graphSamples) {
        cat("PK_MONITOR_HIERARCHICAL_CLUSTERING\n");

        ## Cluster genes
        if (graphGenes)
            Hclust.func(K,"Genes","GChclustgenes.eps");

        ## Cluster samples
        if (graphSamples)
            Hclust.func(t(K),"Samples","GChclustsamples.eps");
    }

    return();
}


##******************************************************************************
ShowMatrix.func<-function(X,N,p)
{
    rowlblsfile<-paste(GCoutput,"RowLbls.txt",sep="/");
    collblsfile<-paste(GCoutput,"ColLbls.txt",sep="/");
    rowlbls <- scan (rowlblsfile, what="", sep="\n");
    collbls <- scan (collblsfile, what="", sep="\n");
    k<-matrix(X,nrow=N,dimnames=list(rowlbls, collbls));
    x<-c(1:N);
    y<-c(1:p);

    graphTitle = paste("GeneClust","Gene Expression Matrix",sep=" : ");
    matrixepsfile<-paste(GCoutput,"GCMatrix.eps",sep="/");
    if (GCinterpreter == "R")
    {
        NewGraph.func(graphTitle, matrixepsfile);
        matrixgraph<-image(x,y,k,axes=FALSE,col=ColorSchemeImages.func());
        legend(k,x=1.045*N,y=p,hor=F,cex=0.65);
        par (cex.axis=0.2,las=2);
        axis(1,at=x,labels=dimnames(k)[[1]],adj=1,col=1);
        axis(2,at=y,labels=dimnames(k)[[2]],adj=1,col=1);
	par(cex.axis=1,las=1);
    }
    else {
        Motif(graphTitle);
        matrixgraph<-image(x,y,k,axes=FALSE);
        image.legend(k,x=1.045*N,y=p,size=c(0.085,4.2),hor=F,cex=0.65);
        axis(1,at=x,labels=dimnames(k)[[1]],adj=1,col=1,cex=0.45,srt=90);
        axis(2,at=y,labels=dimnames(k)[[2]],adj=1,col=1,cex=0.45);
        print(matrixgraph);
    }

    return();
}

InitFigure.func <- function (figuretitle,screenlayout,filename)
{
    windowName = paste("SPLUS",figuretitle,sep=" : ");
    NewGraph.func(windowName, filename);

    scx <- split.screen (figs=matrix(c(0,1,0.9,1, 0,1,0,0.9), nrow = 2, ncol=4, byrow=TRUE));
    screen(scx[1]);
    text(0.5, 0.5, figuretitle, cex=2.0);
    scy <- split.screen (figs=screenlayout, screen=scx[2]);
    return (scy);
}

##******************************************************************************
## npar - number of screen partitions
##******************************************************************************
ShowClusters.func<-function(GCMatrix, screenlayout)
{
    nCol <- ncol (GCMatrix);
    p <- nCol;
    ## Shows all clusters on the same graph
    index1<-1;
    w<-8.15;
    h<-2.5;
    basetitle<-"GeneShave Cluster Plots";

    ## Open "clusters.txt" file and extract cluster filenames from it
    clustersfile<- paste(GCoutput,"clusters.txt",sep="/");
    clusterlist <- scan(clustersfile, what="", sep="\n");
    numclusters <- length (clusterlist);
    stringmatrix<-matrix(clusterlist, numclusters,byrow=FALSE);

    ## Open "vlabels.txt" and extract the variance filenames
    varlabelsfile<-paste(GCoutput,"vlabels.txt",sep="/");
    varlabels<-matrix(scan(varlabelsfile,what="",n=numclusters,sep="\n"),
                           numclusters,byrow=FALSE);
    ## Open "clabels.txt" file and extract the cluster file names
    labelsfile<-paste(GCoutput,"clabels.txt",sep="/");
    genelabels<-matrix(scan(labelsfile,what="",n=numclusters,sep="\n"),
                            numclusters,byrow=FALSE);
    ## Open "elabels.txt" file and extract the eigenvalue file names
    elabelsfile<-paste(GCoutput,"elabels.txt",sep="/");
    elabels<-matrix(scan(elabelsfile,what="",n=numclusters,sep="\n"),
                         numclusters,byrow=FALSE);
    ## Open "pclabels.txt" file and extract the pc vector file names
    pclabelsfile<-paste(GCoutput,"pclabels.txt",sep="/");
    pclabels<-matrix(scan(pclabelsfile,what="",n=numclusters,sep="\n"),
                          numclusters,byrow=FALSE);
    
    ## Open "clusterlength.txt" file and extract a matrix with first
    ## column optimal cluster index and second column number of genes
    ## per cluster
    clusterlengthfile<-paste(GCoutput,"clusterlengths.txt",sep="/");
    Ncluster<-matrix(scan(clusterlengthfile),numclusters,byrow=TRUE);
    ## Generate Column labels filename with proper path
    columnfile<-paste(GCoutput,"ColLbls.txt",sep="/");

    nscreen <- prod (screenlayout);
    n<-1;
    while (n <= numclusters) {
        figurenum<- (n-1)%%nscreen + 1;
        if (figurenum == 1) {
            pagenum<- (n-1)%/%nscreen + 1;
            outfile<-paste(GCoutput,"GCclusterplots",sep="/");
            outfile<-paste(outfile,pagenum,".eps",sep="");
	    scy <- InitFigure.func (basetitle, screenlayout, outfile);
        }
	screen (scy[figurenum]);

        myevalue<-paste(GCoutput,elabels[n,1],sep="/");
        evalue<-read.table(file=myevalue,header=FALSE);
        varlabels[n]<-paste(GCoutput,varlabels[n],sep="/");
        variance<-read.table(file=varlabels[n],header=FALSE);
        genelabels[n]<-paste(GCoutput,genelabels[n],sep="/");
        stringmatrix[n]<-paste(GCoutput,stringmatrix[n],sep="/");
        k<-matrix(scan(file=stringmatrix[n]),ncol=p,byrow=FALSE);
        k<-matrix(k,nrow=nrow(k),dimnames=list(scan(genelabels[n],what="",
                  n=Ncluster[n,2],sep="\n"),scan(columnfile,what="",
                  n=p,sep="\n")));
        pclabelfile<-paste(GCoutput,pclabels[n,1],sep="/");
        pcvalue<-scan(file=pclabelfile);
        weights<-rep(0,Ncluster[n,2]);
        for (i in 1:Ncluster[n,2]) {
            weights[i]<-sum(k[i,]*pcvalue);
        }

        x<-c(1:Ncluster[n,2]);
        y<-c(1:p);
        ## Display image of cluster sorted w.r.t. column means
        me<-c(rep(0,p));
        for (i in 1:p) {
            me[i]<-mean(k[,i]*weights);
        }
	#cat ("Weights: ", weights[sort.list(weights)], "\n");
	#cat ("me: ", me[sort.list(me)], "\n");
        k1<-k[sort.list(weights),sort.list(me)];

        xloc<-length(dimnames(k1)[[2]]);
        yloc<-length(dimnames(k1)[[1]]);
        if (GCinterpreter == "R") {
            image(t(k1),axes=FALSE,col=ColorSchemeImages.func());
            legend(k1,x=1.045*xloc,y=yloc,hor=F,cex=0.65);
	}
	else {
            image(t(k1),axes=FALSE);
            image.legend(k1,x=1.045*xloc,y=yloc,size=c(0.085,1.55),hor=F,cex=0.65);
	}
        ## To include axes titles do:
        ##     image(x,y,k,xlab="GENES",ylab="Cell Boundaries");
        ##
        ## The col par sets the colcor of the axis and text. Since I'd like my
        ## background to be dark blue I set it = 0
        ##
        ## cex = .5 defines the relative size of the font used in labels
        if (GCinterpreter == "R") {

	    # Draw axis right-justified in normal text.

	    op <- par (adj=1, col=1);
	    pin = par()$pin;  # pin[1] is width, pin[2] is height
	    cxy = par()$cxy;  # cxy[1] is width, cxy[2] is height
	    fig = par()$fig;  # cxy[1] is width, cxy[2] is height
	    plt = par()$plt;
	    cat ("pin is ", pin[1], " by ", pin[2], "\n");
	    cat ("cxy is ", cxy[1], " by ", cxy[2], "\n");
	    cat ("cin/pin is ", par()$cin/par()$pin, "\n");
	    cat ("fig is ", fig, "\n");
	    cat ("plt is ", par()$plt, "\n");
	    cat ("p=",p, " Ncluster=",Ncluster[n,2],"\n");
	    maxw1 = max(strwidth(dimnames(k1)[[2]]));
	    maxh1 = max(strheight(dimnames(k1)[[2]]));
	    maxw = max(strwidth(dimnames(k1)[[1]]));
	    maxh = max(strheight(dimnames(k1)[[1]]));
	    cat("maxw1=",maxw1," maxh1=",maxh1,"\n");
	    cat("maxw2=",maxw ," maxh2=",maxh ,"\n");

	    # Try to guess "good" values of cex.axis so that the axis labels
	    # are readable.  Sometimes R simply does not display the axes or
	    # the title/subtitles, probably because cex.axis is too large.
	    # However, calculating cex.axis seems a lot more difficult than
	    # it "should" be.
	    # If the axes/subtitles aren't display, try reducing cex.axis.

	    # Draw left axis
	    height1row= (plt[4]-plt[3])*(fig[4]-fig[3])/Ncluster[n,2];
	    cat("Height1row=", (plt[4]-plt[3])*(fig[4]-fig[3]), "/", Ncluster[n,2], "\n");
	    cexaxis = min (0.9, height1row/maxh);
	    axiswidth = plt[1] * (fig[2]-fig[1]);
	    cexaxis = min (cexaxis, axiswidth/maxw);
	    cat("cexaxis2 = min(", 0.9,axiswidth/maxw,height1row/maxh,")\n");
	    if (displayName == "X11") {
	        cexaxisold=0.2;
	    }
	    else {
	        cexaxisold=min(0.5,1/maxw,pin[2]/Ncluster[n,2]);
	    }
            axis(2,at=(x-1)/(Ncluster[n,2]-1),labels=dimnames(k1)[[1]],cex.axis=cexaxis,las=2);

	    # Draw bottom axis
	    width1col= (plt[2]-plt[1]) * (fig[2]-fig[1])/p;
	    cat("Width1col=", (plt[2]-plt[1])*(fig[2]-fig[1]),"/",p, "\n");
	    cexaxis = min (0.9, width1col/maxh1);
	    axiswidth = plt[3] * (fig[4]-fig[3]);
	    cexaxis = min (cexaxis, axiswidth/maxw1);
	    cat("cexaxis1 = min(", 0.9,axiswidth/maxw1,width1col/maxh1,")\n");
	    if (displayName == "X11") {
	        cexaxisold=0.3;
	    }
	    else {
	        cexaxisold=min(0.5,0.8/maxh1,pin[1]/p);
	    }
            axis(1,at=(y-1)/(p-1),labels=dimnames(k1)[[2]],cex.axis=cexaxis,las=2);

	    box();
        }
        else {
            axis(2,at=x,labels=dimnames(k1)[[1]], adj=1, col= 1, cex= .5);
            axis(1,at=y,labels=dimnames(k1)[[2]], srt=90, adj= 1, col= 1, cex= .5);
        }
       
        optindex<-Ncluster[n,1];
        percvariance<-variance[optindex,2];
        tvariance<-variance[optindex,5];
        bvariance<-variance[optindex,4];
        relvariance<-variance[optindex,4]/variance[optindex,6];
        clustertitle<-paste("Cluster #",n);
        clustersubtitle<-paste(" eigenvalue=",round(evalue,4),
                               " %variance=",round(percvariance,4),
                               " VT=",round(tvariance,4),
                               " VB=",round(bvariance,4),
                               " VB/VW=",round(relvariance,4));
	par (cex = 1.0, adj = 0.5);
        title(main=clustertitle,sub=clustersubtitle);
	par(op);

        if ((n%%2) == 0 || n == numclusters) {
            if (GCinterpreter == "R") {
            }
            else {
            printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }
}


##******************************************************************************
ShowGap.func <- function()
{
    ## Shows all clusters on the same graph
    index1<-1;
    basetitle<-"GeneShave Gap Curve Graphs";
    gaplabelfile<- paste(GCoutput,"glabels.txt",sep="/");
    gaplabellist<- scan(gaplabelfile,what="",sep="\n");
    numclusters <- length(gaplabellist);
    gaplabels   <- matrix(gaplabellist, numclusters,byrow=FALSE);

    screenlayout = c(2,2);
    npar <- prod(screenlayout);
    n<-1;
    while (n <= numclusters) {
        figurenum<-(n-1)%%npar + 1;
        if (figurenum == 1) {
            pagenum<- (n-1)%/%npar + 1;
            outfile<-paste(GCoutput,"GCgapplots",sep="/");
            outfile<-paste(outfile,pagenum,".eps",sep="");
	    scy <- InitFigure.func (basetitle, screenlayout, outfile);
        }
	screen (scy[figurenum]);

        gapfile<-paste(GCoutput,gaplabels[n,1],sep="/");
        PlotGap.func(gapfile);
        clustertitle<-paste("Cluster #",n);
        title(main=clustertitle,cex=1.0);
        if ((n%%npar) == 0 || n == numclusters) {
            ## maintitle<-paste("\n",basetitle,sep="");
            ## mtext(maintitle,side=3,outer=TRUE,cex=1.4,font=2);
            ## Save graphs to postscript file
            if (GCinterpreter == "R") {
            }
            else {
                printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }

    return();
}


##******************************************************************************
ShowPC.func<-function()
{
    ## Shows all clusters on the same graph
    n<-1;
    index1<-1;
    basetitle<-"GeneShave Principal Component Graphs";
    pclabelfile<-paste(GCoutput,"pclabels.txt",sep="/");
    pclabellist <- scan(pclabelfile,what="",sep="\n");
    numclusters <- length(pclabellist);
    pclabels<-matrix(pclabellist, numclusters,byrow=FALSE);

    screenlayout = c(2,2);
    npar<-prod(screenlayout);
    while (n <= numclusters) {
        figurenum <- (n-1)%%npar+1;
        if (figurenum == 1) {
            pagenum<- (n-1)%/%npar + 1;
            outfile<-paste(GCoutput,"GCpcplots",sep="/");
            outfile<-paste(outfile,pagenum,".eps",sep="");
	    scy <- InitFigure.func (basetitle, screenlayout, outfile);
        }
	screen (scy[figurenum]);

        pcfile<-paste(GCoutput,pclabels[n,1],sep="/");
        PlotPC.func(pcfile);
        clustertitle<-paste("Cluster #",n);
        title(main=clustertitle,cex=1.0);
        if ((n%%npar) == 0 || n == numclusters) {
            ##maintitle<-paste("\n",basetitle,sep="");
            ##mtext(maintitle,side=3,outer=TRUE,cex=1.4,font=2);
            ## Save graphs to postscript file
            if (GCinterpreter == "R") {
            }
            else {
                printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }

    return();
}


##******************************************************************************
ShowVariance.func<-function()
{
    ## Shows all clusters on the same graph
    index1<-1;
    basetitle<-"GeneShave Variance Graphs";
    varlabelsfile<-paste(GCoutput,"vlabels.txt",sep="/");
    varlabelslist <- scan(varlabelsfile,what="",sep="\n");
    numclusters <- length(varlabelslist);
    varlabels <- matrix(varlabelslist, numclusters,byrow=FALSE);

    screenlayout = c(2,2);
    npar<-4;
    n<-1;
    while (n <= numclusters) {
        figurenum <- (n-1)%%npar + 1;
        if (figurenum == 1) {
            pagenum<- (n-1)%/%npar + 1;
            outfile<-paste(GCoutput,"GCvarianceplots",sep="/");
            outfile<-paste(outfile,pagenum,".eps",sep="");
	    scy <- InitFigure.func (basetitle, screenlayout, outfile);
        }
	screen (scy[figurenum]);

        varfile<-paste(GCoutput,varlabels[n,1],sep="/");
        PlotVariance.func(varfile);
        clustertitle<-paste("Cluster #",n);
        title(main=clustertitle,cex=1.0);
        if (GCinterpreter == "R") {
        }
        else {
            key(text=list(c("Real Data","Randomized Data"),col=1:2, adj=1),
                col=c(1,2),type=c("p","p"),points=list(pch=0:1));
	}
        if ((n%%npar) == 0 || n == numclusters) {
            ##maintitle<-paste("\n",basetitle,sep="");
            ##mtext(maintitle,side=3,outer=TRUE,cex=1.4,font=2);
            ## Save graphs to postscript file
            if (GCinterpreter == "R") {
            }
            else {
                printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }

    return();
}


##*****************************************************************************
DisplayGraphs.func<-function(GCMatrix,flagNoise,SNR)
{
    ## display the original matrix
    nRow <- nrow (GCMatrix);
    nCol <- ncol (GCMatrix);
    matrixgraph<-ShowMatrix.func(GCMatrix,nRow,nCol);
    if (flagNoise < 0) {
        maintitle<-"Observed Gene Expression Matrix";
        subtitle<-paste("# Genes =",nRow,"# cell-lines=",nCol);
        title(main=maintitle,sub=subtitle);
    }
    else if (flagNoise == 0) {
        maintitle<-"Gene Expression Matrix, SNR= infinity";
        subtitle<-paste("# Genes =",nRow,"# cell-lines=",nCol);
        title(main=maintitle,sub=subtitle);
        outfile<-paste(GCoutput,"GCmatrix.ps",sep="/");
        if (GCinterpreter == "R") {
        }
        else {
        printgraph(horizontal=TRUE,print=FALSE,file=outfile);
	}
    }
    else {
        sqrtSNR<-sqrt(SNR);
        maintitle<-paste("Gene Expression Matrix, SNR=",round(sqrtSNR,4));
        subtitle<-paste("# Genes =",nRow,"# cell-lines=",nCol);
        title(main=maintitle,sub=subtitle);
        outfile<-paste(GCoutput,"GCmatrix.ps",sep="/");
        if (GCinterpreter == "R") {
        }
        else {
        printgraph(horizontal=TRUE,print=FALSE,file=outfile);
	}
    }
    cat ("DisplayGraphs.func completed\n");
}

DisplayClusterGraphs.func <- function(GCMatrix)
{
    ## display clusters
    ShowClusters.func(GCMatrix, c(2,1));

    ## display gap
    ShowGap.func();

    ## display variance
    ShowVariance.func();

    ## display principal component
    ShowPC.func();
}


GCDisplayData.func<-function()
{
    ## update user interface
    cat("PK_MONITOR_READ_MATRIX\n");

    matrixfile <- paste(GCoutput,"GCMatrix.dat",sep="/");
    load (file=matrixfile);

    cat("PK_MONITOR_DISPLAY_RESULTS\n");
    ## show data matrix, if requested
    if (graphDataMatrix)
        DisplayGraphs.func(data$C, -1, 0);

    ## show hierarchical clustering, if requested
    if (graphGenes || graphSamples)
        DisplayHclust.func(data$C);

    ## show gene shaving clusters, if requested
    if (graphShaving)
        DisplayClusterGraphs.func(data$C);

    cat("PK_MONITOR_APP_PROCESSING_COMPLETE\n");
}

