##################### FILE ABSTRACT ############################################
##
## Synopsis::         Source for graphics routines
##
## File Name::        %M%
## Version::          %I%
## Type::             source
## Date Completed::   %G%
##
## Environment::
##     Language::     Splus
##     Computer::     Compaq/Alpha
##     OS::           DigitalUNIX
##     OSVersion::    4.0
##
## Module Information::
##     Owner::        MDACC
##     Department::   Biostatistics
##     Researcher::   Kim-Anh Do
##     Application::  geneclust
##
################################################################################


##******************************************************************************
InitGraphics.func<-function()
{
    InitPostscriptOptions.func<-function()
    {
        geneclust.background<-xgetrgb("background");
        geneclust.lines<-xgetrgb("lines");
        geneclust.text<-xgetrgb("text");
        geneclust.polygons<-xgetrgb("polygons");
        geneclust.images<-xgetrgb("images");
        ps.options(background=geneclust.background,
                   image.colors=geneclust.images,
                   colors=geneclust.lines);
        return ();
    }

    Motif("Catch22", iconic = T);
    InitPostscriptOptions.func();
    dev.off();
}


##******************************************************************************
InitSplusColorScheme.func<-function()
{
    splus.colorscheme.name      <-'name : "GeneClust";';
    splus.colorscheme.background<-'background : white;';
    splus.colorscheme.lines     <-'lines : black blue;';
    splus.colorscheme.text      <-'text : blue;';
    splus.colorscheme.polygons  <-'polygons : blue;';
    lscale <- c('0','0','0','1','1','2','2','3');
    scale  <- c('8','9','a','b','c','d','e','f');
    redscale <- paste("#",scale,rev(lscale),rev(lscale),sep="");
    grnscale <- paste("#",lscale,rev(scale),lscale,sep="");
    splus.colorscheme.images    <- paste('images : ',
                                         paste(grnscale,collapse=' '),
                                         '#662',
                                         '#842',
                                         paste(redscale,collapse=' '));

    return (paste('sgraphMotif.colorSchemes :',
                  splus.colorscheme.name,
                  splus.colorscheme.background,
                  splus.colorscheme.lines,
                  splus.colorscheme.text,
                  splus.colorscheme.polygons,
                  splus.colorscheme.images,
                  sep=" "));
}


##******************************************************************************
Motif<-function(windowName = "SPLUS", ..., iconic = F)
{
    splus.colorscheme<-InitSplusColorScheme.func();
    splus.defaultcolorscheme<-'sgraphMotif.defaultColorScheme : 1';

    X11.options.iconic<-"-iconic";
    X11.options.title<-paste("-title '",windowName,"'",sep="");
    X11.options.colorscheme<-paste("-xrm '",
                                   splus.colorscheme,
                                   "'",
                                   sep="");
    X11.options.defaultcolorscheme<-paste("-xrm '",
                                          splus.defaultcolorscheme,
                                          "'",
                                          sep="");

    if (iconic)
    {
        X11.options<-c(X11.options.title,
                       X11.options.iconic,
                       X11.options.colorscheme,
                       X11.options.defaultcolorscheme);
    }
    else
    {
        X11.options<-c(X11.options.title,
                       X11.options.colorscheme,
                       X11.options.defaultcolorscheme);
    }
    options<-X11.options;

    #
    # The rest is straight from the motif() function definition. Had problems
    # with the default colorscheme overriding our specified one.
    #
    n<-length(options);
    tmp<-rep(" ", 2 * n);
    tmp[(1:n) * 2 - 1]<-options;
    tpar<-as.single(AsciiToInt(tmp));
    pipegraph(name="motif",pars=as.single(tpar),npars=length(tpar), ...);
    par(tck = (-0.01));
    invisible();

    return ();
}

